module v386
(
    // Тактовый генератор
    input               clock,
    input               reset_n,
    input               locked,
    // Магистраль данных 8 битная
    output      [31:0]  address,
    input       [ 7:0]  in,
    output reg  [ 7:0]  out,
    output reg          we
);

`include "v386_decl.v"
`include "v386_alu.v"

always @(posedge clock)
if (locked) begin // Процессор работает
if (reset_n == 1'b0) begin

    t           <= fetch;
    cs          <= 16'hF800; // 16'hF000
    eip         <= 16'h0000; // 16'hFFF0
    __segment   <= 80'h0000;
    __adsize    <= 1'b0;
    __opsize    <= 1'b0;
    __override  <= 1'b0;
    __rep       <= 1'b0;
    src         <= 1'b0; // Указатель на CS:EIP
    prot        <= 1'b0; // Процессор работает в REAL MODE

end else case (t)

// Считывание опкода или префикса
fetch: begin

    eip     <= eip_next;
    opcode  <= in;
    size    <= in[0];
    dir     <= in[1];
    alu     <= in[5:3];
    regn    <= in[2:0];
    fn      <= 1'b0;
    fn2     <= 1'b0;
    ignoreo <= 1'b0;
    t_next  <= fetch;

    case (in)
    // Сегментные префиксы, 80-разрядные
    8'h26: begin __segment <= es; __override <= 1'b1; end
    8'h2E: begin __segment <= cs; __override <= 1'b1; end
    8'h36: begin __segment <= ss; __override <= 1'b1; end
    8'h3E: begin __segment <= ds; __override <= 1'b1; end
    8'h64: begin __segment <= fs; __override <= 1'b1; end
    8'h65: begin __segment <= gs; __override <= 1'b1; end
    8'h66: begin __opsize <= ~__opsize; end
    8'h67: begin __adsize <= ~__adsize; end
    8'hF0, 8'h9B, 8'h90: begin /* fwait, lock, nop */ end
    8'hF2, 8'hF3: __rep <= in[1:0];
    // Исполнение опкода
    default: begin

        t <= exec;

        // Защелкивание префиксов
        rep         <= __rep;       __rep       <= 2'b00;
        override    <= __override;  __override  <= 1'b0;
        opsize      <= __opsize;    __opsize    <= defsize;
        adsize      <= __adsize;    __adsize    <= defsize;
        segment     <= __segment;   __segment   <= ds;

        casex (in)
        8'b00xx_x0xx: begin t <= fetch_modrm; end
        endcase

    end
    endcase

end

// Считывание modrm
fetch_modrm: case (fn2)
0: begin

    modrm   <= in;
    eip     <= eip_next;
    ea      <= 1'b0;

    // Левый операнд
    case (dir ? in[5:3] : in[2:0])
    0: op1 <= size ? (opsize ? eax : eax[15:0]) : eax[ 7:0];
    1: op1 <= size ? (opsize ? ecx : ecx[15:0]) : ecx[ 7:0];
    2: op1 <= size ? (opsize ? edx : edx[15:0]) : edx[ 7:0];
    3: op1 <= size ? (opsize ? ebx : ebx[15:0]) : ebx[ 7:0];
    4: op1 <= size ? (opsize ? esp : esp[15:0]) : eax[15:8];
    5: op1 <= size ? (opsize ? ebp : ebp[15:0]) : ecx[15:8];
    6: op1 <= size ? (opsize ? esi : esi[15:0]) : edx[15:8];
    7: op1 <= size ? (opsize ? edi : edi[15:0]) : ebx[15:8];
    endcase

    // Правый операнд
    case (dir ? in[2:0] : in[5:3])
    0: op2 <= size ? (opsize ? eax : eax[15:0]) : eax[ 7:0];
    1: op2 <= size ? (opsize ? ecx : ecx[15:0]) : ecx[ 7:0];
    2: op2 <= size ? (opsize ? edx : edx[15:0]) : edx[ 7:0];
    3: op2 <= size ? (opsize ? ebx : ebx[15:0]) : ebx[ 7:0];
    4: op2 <= size ? (opsize ? esp : esp[15:0]) : eax[15:8];
    5: op2 <= size ? (opsize ? ebp : ebp[15:0]) : ecx[15:8];
    6: op2 <= size ? (opsize ? esi : esi[15:0]) : edx[15:8];
    7: op2 <= size ? (opsize ? edi : edi[15:0]) : ebx[15:8];
    endcase

    // 32-bit MODRM
    if (adsize) begin

        case (in[2:0])
        3'b000: ea <= eax;
        3'b001: ea <= ecx;
        3'b010: ea <= edx;
        3'b011: ea <= ebx;
        3'b100: ea <= 0;
        3'b101: ea <= ^in[7:6] ? ebp : 0;
        3'b110: ea <= esi;
        3'b111: ea <= edi;
        endcase

        case (in[7:6])
        2'b00: begin

            if      (in[2:0] == 3'b101) fn2 <= 1;  // DISP32
            else if (in[2:0] == 3'b100) fn2 <= 10; // SIB
            else begin

                fn2 <= 4;
                src <= 1'b1;
                if (ignoreo) begin t <= exec; fn2 <= 0; end

            end

        end
        2'b01: fn2 <= in[2:0] == 3'b100 ? 10 : 3; // 8     bit | SIB
        2'b10: fn2 <= in[2:0] == 3'b100 ? 10 : 1; // 16/32 bit | SIB
        2'b11: begin fn2 <= 0; t <= exec; end
        endcase

        if (!override && (^in[7:6] && in[2:0] == 3'b101))
            segment <= ss;

    end
    // 16-bit MODRM
    else begin

        case (in[2:0])
        3'b000: ea[15:0] <= ebx + esi;
        3'b001: ea[15:0] <= ebx + edi;
        3'b010: ea[15:0] <= ebp + esi;
        3'b011: ea[15:0] <= ebp + edi;
        3'b100: ea[15:0] <= esi;
        3'b101: ea[15:0] <= edi;
        3'b110: ea[15:0] <= ^in[7:6] ? ebp : 1'b0;
        3'b111: ea[15:0] <= ebx;
        endcase

        // Выбор сегмента по умолчанию
        if (!override && (in[2:1] == 2'b01 || (^in[7:6] && in[2:0] == 3'b110)))
            segment <= ss;

        // Выбор решения
        case (in[7:6])
        2'b00: begin

            // Читать +disp16
            if (in[2:0] == 3'b110) fn2 <= 1;
            // Сразу читать операнды из памяти
            else begin

                fn2 <= 4;
                src <= 1'b1;
                if (ignoreo) begin t <= exec; fn2 <= 0; end

            end

        end
        2'b01: fn2 <= 3; // 8 bit
        2'b10: fn2 <= 1; // 16 bit
        2'b11: begin fn2 <= 0; t <= exec; end
        endcase

    end

end

// DISP16/32
1: begin fn2 <= 2; ea <= ea + in; eip <= eip_next; end
2: begin

    fn2      <=  adsize ? 8 : 4;
    src      <= !adsize;
    ea[31:8] <= ea[31:8] + in;
    eip      <= eip_next;

    if (ignoreo && !adsize) begin t <= exec; fn2 <= 0; end

end

// DISP8
3: begin

    fn2 <= 4;
    ea  <= ea + {{24{in[7]}}, in};
    src <= 1'b1;
    eip <= eip_next;

    if (ignoreo) begin t <= exec; fn2 <= 0; end

end

// OPERAND-7:0
4: begin

    if (dir) op2 <= in; else op1 <= in;
    if (size) begin fn2 <= 5; ea <= ea + 1; end
    else      begin fn2 <= 0; t  <= exec; end

end

// OPERAND-15:8
5: begin

    if (dir) op2[15:8] <= in; else op1[15:8] <= in;
    if (opsize) begin fn2 <= 6; ea <= ea + 1; end
    else        begin fn2 <= 0; ea <= ea - 1; t <= exec;  end

end

// OPERAND-23:16
6: begin

    fn2 <= 7; ea <= ea + 1;
    if (dir) op2[23:16] <= in; else op1[23:16] <= in;

end

// OPERAND-31:24
7: begin

    t  <= exec;
    fn2 <= 0; ea <= ea - 3;
    if (dir) op2[31:24] <= in; else op1[31:24] <= in;

end

8: begin fn2 <= 9; ea[31:16] <= ea[31:16] + in; eip <= eip_next; end
9: begin

    fn2         <= 4;
    ea[31:24]   <= ea[31:24] + in;
    src         <= 1'b1;
    eip         <= eip_next;

    if (ignoreo) begin t <= exec; fn2 <= 0; end

end

10: begin

    case (in[5:3])
    3'b000: ea <= sib_base + (eax << in[7:6]);
    3'b001: ea <= sib_base + (ecx << in[7:6]);
    3'b010: ea <= sib_base + (edx << in[7:6]);
    3'b011: ea <= sib_base + (ebx << in[7:6]);
    3'b100: ea <= sib_base;
    3'b101: ea <= sib_base + (ebp << in[7:6]);
    3'b110: ea <= sib_base + (esi << in[7:6]);
    3'b111: ea <= sib_base + (edi << in[7:6]);
    endcase

    case (modrm[7:6])
    2'b00: if (in[2:0] == 3'b101)
           begin fn2 <= 1; end
    else   begin fn2 <= 4; src <= 1'b1; end
    2'b01: begin fn2 <= 3; end // disp8
    2'b10: begin fn2 <= 1; end // disp32
    2'b11: begin fn2 <= 0; t <= exec; end
    endcase

    if (!override && ((^modrm[7:6] && in[2:0] == 3'b101) || (in[5:3] == 3'b101)))
        segment <= ss;

    if (modrm[7:6] == 2'b00 && in[2:0] != 3'b101 && ignoreo) begin t <= exec; fn2 <= 0; end

end

endcase

endcase
end

endmodule
